@@ -4,8 +4,13 @@ import copy
 import re
 import warnings
 
-
-from collections import MutableMapping
+try:
+    from collections.abc import MutableMapping
+except ImportError:
+    try:
+        from collections import MutableMapping
+    except ImportError:
+        from typing import MutableMapping
 from functools import lru_cache
 
 from .mask import Mask
